@@ -1,13 +1,34 @@
 package Sys::Syslog;
+use strict;
+use Carp;
 require 5.006;
 require Exporter;
-use Carp;
-use strict;
 
+our $VERSION = '0.11';
 our @ISA = qw(Exporter);
-our @EXPORT = qw(openlog closelog setlogmask syslog);
-our @EXPORT_OK = qw(setlogsock);
-our $VERSION = '0.09';
+
+our %EXPORT_TAGS = (
+    standard => [qw(openlog syslog closelog setlogmask)],
+    extended => [qw(setlogsock)],
+    macros => [qw(
+        LOG_ALERT LOG_AUTH LOG_AUTHPRIV LOG_CONS LOG_CRIT LOG_CRON
+        LOG_DAEMON LOG_DEBUG LOG_EMERG LOG_ERR LOG_FACMASK LOG_FTP
+        LOG_INFO LOG_KERN LOG_LFMT LOG_LOCAL0 LOG_LOCAL1 LOG_LOCAL2
+        LOG_LOCAL3 LOG_LOCAL4 LOG_LOCAL5 LOG_LOCAL6 LOG_LOCAL7 LOG_LPR
+        LOG_MAIL LOG_NDELAY LOG_NEWS LOG_NFACILITIES LOG_NOTICE
+        LOG_NOWAIT LOG_ODELAY LOG_PERROR LOG_PID LOG_PRIMASK LOG_SYSLOG
+        LOG_USER LOG_UUCP LOG_WARNING
+    )],
+);
+
+our @EXPORT = (
+    @{$EXPORT_TAGS{standard}}, 
+);
+
+our @EXPORT_OK = (
+    @{$EXPORT_TAGS{extended}}, 
+    @{$EXPORT_TAGS{macros}}, 
+);
 
 # it would be nice to try stream/unix first, since that will be
 # most efficient. However streams are dodgy - see _syslog_send_stream
@@ -24,16 +45,21 @@ my $fail_time = undef;
 our ($connected, @fallbackMethods, $syslog_send, $host);
 
 use Socket ':all';
-use Sys::Hostname;
+use POSIX qw(strftime setlocale LC_TIME);
 
 =head1 NAME
 
 Sys::Syslog - Perl interface to the UNIX syslog(3) calls
 
+=head1 VERSION
+
+Version 0.11
+
 =head1 SYNOPSIS
 
-    use Sys::Syslog;                          # all except setlogsock, or:
-    use Sys::Syslog qw(:DEFAULT setlogsock);  # default set, plus setlogsock
+    use Sys::Syslog;                          # all except setlogsock(), or:
+    use Sys::Syslog qw(:DEFAULT setlogsock);  # default set, plus setlogsock()
+    use Sys::Syslog qw(:standard :macros);    # standard functions, plus macros
 
     setlogsock $sock_type;
     openlog $ident, $logopt, $facility;       # don't forget this
@@ -41,87 +67,185 @@ Sys::Syslog - Perl interface to the UNIX syslog(3) calls
     $oldmask = setlogmask $mask_priority;
     closelog;
 
+
 =head1 DESCRIPTION
 
-Sys::Syslog is an interface to the UNIX C<syslog(3)> program.
+C<Sys::Syslog> is an interface to the UNIX C<syslog(3)> program.
 Call C<syslog()> with a string priority and a list of C<printf()> args
 just like C<syslog(3)>.
 
-Syslog provides the functions:
+
+=head1 EXPORTS
+
+C<Sys::Syslog> exports the following C<Exporter> tags: 
 
 =over 4
 
-=item openlog $ident, $logopt, $facility
+=item *
+
+C<:standard> exports the standard C<syslog(3)> functions: 
+
+    openlog closelog setlogmask syslog
+
+=item *
+
+C<:extended> exports the Perl specific functions for C<syslog(3)>: 
+
+    setlogsock
+
+=item *
+
+C<:macros> exports the symbols corresponding to most of your C<syslog(3)> 
+macros. See L<"CONSTANTS"> for the supported constants and their meaning. 
+
+=back
+
+By default, C<Sys::Syslog> exports the symbols from the C<:standard> tag. 
+
+
+=head1 FUNCTIONS
+
+=over 4
+
+=item B<openlog($ident, $logopt, $facility)>
 
 Opens the syslog.
-I<$ident> is prepended to every message.  I<$logopt> contains zero or
-more of the words I<pid>, I<ndelay>, I<nowait>.  The cons option is
+C<$ident> is prepended to every message.  C<$logopt> contains zero or
+more of the words C<pid>, C<ndelay>, C<nowait>.  The C<cons> option is
 ignored, since the failover mechanism will drop down to the console
-automatically if all other media fail.  I<$facility> specifies the
-part of the system to report about, for example LOG_USER or LOG_LOCAL0:
+automatically if all other media fail.  C<$facility> specifies the
+part of the system to report about, for example C<LOG_USER> or C<LOG_LOCAL0>:
 see your C<syslog(3)> documentation for the facilities available in
-your system. This function will croak if it can't connect to the syslog
-daemon.
+your system. Facility can be given as a string or a numeric macro. 
+
+This function will croak if it can't connect to the syslog daemon.
+
+Note that C<openlog()> now takes three arguments, just like C<openlog(3)>.
 
 B<You should use openlog() before calling syslog().>
 
-=item syslog $priority, $message
+B<Options>
 
-=item syslog $priority, $format, @args
+=over 4
 
-If I<$priority> permits, logs I<$message> or I<sprintf($format, @args)>
-with the addition that I<%m> in $message or $format is replaced with
-C<"$!"> (the latest error message).
+=item *
 
-If you didn't use openlog() before using syslog(), syslog will try to
-guess the I<$ident> by extracting the shortest prefix of I<$format>
-that ends in a ":".
+C<ndelay> - Open the connection immediately (normally, the connection is
+opened when the first message is logged).
 
-Note that Sys::Syslog version v0.07 and older passed the $message as
-the formatting string to sprintf() even when no formatting arguments
-were provided.  If the code calling syslog() might execute with older
-versions of this module, make sure to call the function as
-syslog($priority, "%s", $message) instead of syslog($priority,
-$message).  This protects against hostile formatting sequences that
+=item *
+
+C<nowait> - Don't wait for child processes that may have been created 
+while logging the message.  (The GNU C library does not create a child
+process, so this option has no effect on Linux.)
+
+=item *
+
+C<pid> - Include PID with each message.
+
+=back
+
+B<Examples>
+
+Open the syslog with options C<ndelay> and C<pid>, and with facility C<LOCAL0>: 
+
+    openlog($name, "ndelay,pid", "local0");
+
+Same thing, but this time using the macro corresponding to C<LOCAL0>: 
+
+    openlog($name, "ndelay,pid", LOG_LOCAL0);
+
+
+=item B<syslog($priority, $message)>
+
+=item B<syslog($priority, $format, @args)>
+
+If C<$priority> permits, logs C<$message> or C<sprintf($format, @args)>
+with the addition that C<%m> in $message or C<$format> is replaced with
+C<"$!"> (the latest error message). 
+
+C<$priority> can specify a level, or a level and a facility.  Levels and 
+facilities can be given as strings or as macros.
+
+If you didn't use C<openlog()> before using C<syslog()>, C<syslog()> will 
+try to guess the C<$ident> by extracting the shortest prefix of 
+C<$format> that ends in a C<":">.
+
+B<Examples>
+
+    syslog("info", $message);           # informational level
+    syslog(LOG_INFO, $message);         # informational level
+
+    syslog("info|local0", $message);        # information level, Local0 facility
+    syslog(LOG_INFO|LOG_LOCAL0, $message);  # information level, Local0 facility
+
+=over 4
+
+=item B<Note>
+
+C<Sys::Syslog> version v0.07 and older passed the C<$message> as the 
+formatting string to C<sprintf()> even when no formatting arguments
+were provided.  If the code calling C<syslog()> might execute with 
+older versions of this module, make sure to call the function as
+C<syslog($priority, "%s", $message)> instead of C<syslog($priority,
+$message)>.  This protects against hostile formatting sequences that
 might show up if $message contains tainted data.
 
-=item setlogmask $mask_priority
+=back
 
-Sets log mask I<$mask_priority> and returns the old mask.
 
-=item setlogsock $sock_type [$stream_location] (added in 5.004_02)
+=item B<setlogmask($mask_priority)>
+
+Sets the log mask for the current process to C<$mask_priority> and 
+returns the old mask.  If the mask argument is 0, the current log mask 
+is not modified.  See L<"Levels"> for the list of available levels. 
+
+B<Examples>
+
+Only log errors: 
+
+    setlogmask(LOG_ERR);
+
+Log critical messages, errors and warnings: 
+
+    setlogmask(LOG_CRIT|LOG_ERR|LOG_WARNING);
+
+
+=item B<setlogsock($sock_type)>
+
+=item B<setlogsock($sock_type, $stream_location)> (added in 5.004_02)
 
 Sets the socket type to be used for the next call to
-C<openlog()> or C<syslog()> and returns TRUE on success,
-undef on failure.
+C<openlog()> or C<syslog()> and returns true on success,
+C<undef> on failure.
 
-A value of 'unix' will connect to the UNIX domain socket (in some
+A value of C<"unix"> will connect to the UNIX domain socket (in some
 systems a character special device) returned by the C<_PATH_LOG> macro
 (if your system defines it), or F</dev/log> or F</dev/conslog>,
 whatever is writable.  A value of 'stream' will connect to the stream
 indicated by the pathname provided as the optional second parameter.
-(For example Solaris and IRIX require 'stream' instead of 'unix'.)
-A value of 'inet' will connect to an INET socket (either tcp or udp,
-tried in that order) returned by getservbyname(). 'tcp' and 'udp' can
-also be given as values. The value 'console' will send messages
-directly to the console, as for the 'cons' option in the logopts in
-openlog().
+(For example Solaris and IRIX require C<"stream"> instead of C<"unix">.)
+A value of C<"inet"> will connect to an INET socket (either C<tcp> or C<udp>,
+tried in that order) returned by C<getservbyname()>. C<"tcp"> and C<"udp"> can
+also be given as values. The value C<"console"> will send messages
+directly to the console, as for the C<"cons"> option in the logopts in
+C<openlog()>.
 
 A reference to an array can also be passed as the first parameter.
 When this calling method is used, the array should contain a list of
 sock_types which are attempted in order.
 
-The default is to try tcp, udp, unix, stream, console.
+The default is to try C<tcp>, C<udp>, C<unix>, C<stream>, C<console>.
 
-Giving an invalid value for sock_type will croak.
+Giving an invalid value for C<$sock_type> will croak.
 
-=item closelog
 
-Closes the log file.
+=item B<closelog()>
+
+Closes the log file and return true on success.
 
 =back
 
-Note that C<openlog> now takes three arguments, just like C<openlog(3)>.
 
 =head1 EXAMPLES
 
@@ -147,6 +271,167 @@ Note that C<openlog> now takes three arguments, just like C<openlog(3)>.
     syslog('info', 'something happened over here');
 
 
+=head1 CONSTANTS
+
+=head2 Facilities
+
+=over 4
+
+=item *
+
+C<LOG_AUTH> - security/authorization messages
+
+=item *
+
+C<LOG_AUTHPRIV> - security/authorization messages (private)
+
+=item *
+
+C<LOG_CRON> - clock daemon (B<cron> and B<at>)
+
+=item *
+
+C<LOG_DAEMON> - system daemons without separate facility value
+
+=item *
+
+C<LOG_FTP> - ftp daemon
+
+=item *
+
+C<LOG_KERN> - kernel messages
+
+=item *
+
+C<LOG_LOCAL0> through C<LOG_LOCAL7> - reserved for local use
+
+=item *
+
+C<LOG_LPR> - line printer subsystem
+
+=item *
+
+C<LOG_MAIL> - mail subsystem
+
+=item *
+
+C<LOG_NEWS> - USENET news subsystem
+
+=item *
+
+C<LOG_SYSLOG> - messages generated internally by B<syslogd>
+
+=item *
+
+C<LOG_USER> (default) - generic user-level messages
+
+=item *
+
+C<LOG_UUCP> - UUCP subsystem
+
+=back
+
+
+=head2 Levels
+
+=over 4
+
+=item *
+
+C<LOG_EMERG> - system is unusable
+
+=item *
+
+C<LOG_ALERT> - action must be taken immediately
+
+=item *
+
+C<LOG_CRIT> - critical conditions
+
+=item *
+
+C<LOG_ERR> - error conditions
+
+=item *
+
+C<LOG_WARNING> - warning conditions
+
+=item *
+
+C<LOG_NOTICE> - normal, but significant, condition
+
+=item *
+
+C<LOG_INFO> - informational message
+
+=item *
+
+C<LOG_DEBUG> - debug-level message
+
+=back
+
+
+=head1 DIAGNOSTICS
+
+=over 4
+
+=item Invalid argument passed to setlogsock
+
+B<(F)> You gave C<setlogsock()> an invalid value for C<$sock_type>. 
+
+=item no connection to syslog available
+
+B<(F)> C<syslog()> failed to connect to the specified socket.
+
+=item stream passed to setlogsock, but %s is not writable
+
+B<(W)> You asked C<setlogsock()> to use a stream socket, but the given 
+path is not writable. 
+
+=item stream passed to setlogsock, but could not find any device
+
+B<(W)> You asked C<setlogsock()> to use a stream socket, but didn't 
+provide a path, and C<Sys::Syslog> was unable to find an appropriate one.
+
+=item tcp passed to setlogsock, but tcp service unavailable
+
+B<(W)> You asked C<setlogsock()> to use a TCP socket, but the service 
+is not available on the system. 
+
+=item syslog: expecting argument %s
+
+B<(F)> You forgot to give C<syslog()> the indicated argument.
+
+=item syslog: invalid level/facility: %s
+
+B<(F)> You specified an invalid level or facility, like C<LOG_KERN> 
+(which is reserved to the kernel). 
+
+=item syslog: too many levels given: %s
+
+B<(F)> You specified too many levels. 
+
+=item syslog: too many facilities given: %s
+
+B<(F)> You specified too many facilities. 
+
+=item syslog: level must be given
+
+B<(F)> You forgot to specify a level.
+
+=item udp passed to setlogsock, but udp service unavailable
+
+B<(W)> You asked C<setlogsock()> to use a UDP socket, but the service 
+is not available on the system. 
+
+=item unix passed to setlogsock, but path not available
+
+B<(W)> You asked C<setlogsock()> to use a UNIX socket, but C<Sys::Syslog> 
+was unable to find an appropriate an appropriate device.
+
+=back
+
+
 =head1 SEE ALSO
 
 L<syslog(3)>
@@ -159,16 +444,19 @@ E<lt>F<larry@wall.org>E<gt>.
 
 UNIX domain sockets added by Sean Robinson
 E<lt>F<robinson_s@sc.maricopa.edu>E<gt> with support from Tim Bunce 
-E<lt>F<Tim.Bunce@ig.co.uk>E<gt> and the perl5-porters mailing list.
+E<lt>F<Tim.Bunce@ig.co.uk>E<gt> and the C<perl5-porters> mailing list.
 
 Dependency on F<syslog.ph> replaced with XS code by Tom Hughes
 E<lt>F<tom@compton.nu>E<gt>.
 
-Code for constant()s regenerated by Nicholas Clark E<lt>F<nick@ccl4.org>E<gt>.
+Code for C<constant()>s regenerated by Nicholas Clark E<lt>F<nick@ccl4.org>E<gt>.
 
 Failover to different communication modes by Nick Williams
 E<lt>F<Nick.Williams@morganstanley.com>E<gt>.
 
+Extracted from core distribution for publishing on the CPAN by 
+SE<eacute>bastien Aperghis-Tramoni E<lt>sebastien@aperghis.netE<gt>.
+
 
 =head1 BUGS
 
@@ -218,22 +506,26 @@ under the same terms as Perl itself.
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
     # XS function.
-    
     my $constname;
     our $AUTOLOAD;
     ($constname = $AUTOLOAD) =~ s/.*:://;
     croak "&Sys::Syslog::constant not defined" if $constname eq 'constant';
     my ($error, $val) = constant($constname);
-    if ($error) {
-	croak $error;
-    }
+	croak $error if $error;
     no strict 'refs';
     *$AUTOLOAD = sub { $val };
     goto &$AUTOLOAD;
 }
 
-require XSLoader;
-XSLoader::load('Sys::Syslog', $VERSION);
+eval {
+    require XSLoader;
+    XSLoader::load('Sys::Syslog', $VERSION);
+    1
+} or do {
+    require DynaLoader;
+    push @ISA, 'DynaLoader';
+    bootstrap Sys::Syslog $VERSION;
+};
 
 our $maskpri = &LOG_UPTO(&LOG_DEBUG);
 
@@ -253,7 +545,7 @@ sub closelog {
 
 sub setlogmask {
     my $oldmask = $maskpri;
-    $maskpri = shift;
+    $maskpri = shift unless $_[0] == 0;
     $oldmask;
 }
  
@@ -278,11 +570,11 @@ sub setlogsock {
 		    last;
 		}
 	    }
-	    croak "stream passed to setlogsock, but could not find any device"
-		unless defined $syslog_path;
+	    carp "stream passed to setlogsock, but could not find any device"
+		unless defined $syslog_path
         }
 	unless (-w $syslog_path) {
-	    croak "stream passed to setlogsock, but $syslog_path is not writable";
+	    carp "stream passed to setlogsock, but $syslog_path is not writable";
 	    return undef;
 	} else {
 	    @connectMethods = ( 'stream' );
@@ -292,21 +584,21 @@ sub setlogsock {
 	    $syslog_path = _PATH_LOG();
 	    @connectMethods = ( 'unix' );
         } else {
-	    croak 'unix passed to setlogsock, but path not available';
+	    carp 'unix passed to setlogsock, but path not available';
 	    return undef;
         }
     } elsif (lc($setsock) eq 'tcp') {
 	if (getservbyname('syslog', 'tcp') || getservbyname('syslogng', 'tcp')) {
 	    @connectMethods = ( 'tcp' );
 	} else {
-	    croak "tcp passed to setlogsock, but tcp service unavailable";
+	    carp "tcp passed to setlogsock, but tcp service unavailable";
 	    return undef;
 	}
     } elsif (lc($setsock) eq 'udp') {
 	if (getservbyname('syslog', 'udp')) {
 	    @connectMethods = ( 'udp' );
 	} else {
-	    croak "udp passed to setlogsock, but udp service unavailable";
+	    carp "udp passed to setlogsock, but udp service unavailable";
 	    return undef;
 	}
     } elsif (lc($setsock) eq 'inet') {
@@ -314,7 +606,7 @@ sub setlogsock {
     } elsif (lc($setsock) eq 'console') {
 	@connectMethods = ( 'console' );
     } else {
-        croak "Invalid argument passed to setlogsock; must be 'stream', 'unix', 'tcp', 'udp' or 'inet'";
+        croak "Invalid argument passed to setlogsock; must be 'stream', 'unix', 'tcp', 'udp' or 'inet'"
     }
     return 1;
 }
@@ -327,16 +619,16 @@ sub syslog {
     our $facility;
     local($facility) = $facility;	# may need to change temporarily.
 
-    croak "syslog: expecting argument \$priority" unless $priority;
-    croak "syslog: expecting argument \$format"   unless $mask;
+    croak "syslog: expecting argument \$priority" unless defined $priority;
+    croak "syslog: expecting argument \$format"   unless defined $mask;
 
     @words = split(/\W+/, $priority, 2);# Allow "level" or "level|facility".
     undef $numpri;
     undef $numfac;
     foreach (@words) {
 	$num = &xlate($_);		# Translate word to number.
-	if (/^kern$/ || $num < 0) {
-	    croak "syslog: invalid level/facility: $_";
+	if ($_ eq 'kern' || $num <= 0) {
+	    croak "syslog: invalid level/facility: $_"
 	}
 	elsif ($num <= &LOG_PRIMASK) {
 	    croak "syslog: too many levels given: $_" if defined($numpri);
@@ -386,7 +678,11 @@ sub syslog {
     $message = @_ ? sprintf($mask, @_) : $mask;
 
     $sum = $numpri + $numfac;
-    my $buf = "<$sum>$whoami: $message\0";
+    my $oldlocale = setlocale(LC_TIME);
+    setlocale(LC_TIME, 'C');
+    my $timestamp = strftime "%b %e %T", localtime;
+    setlocale(LC_TIME, $oldlocale);
+    my $buf = "<$sum>$timestamp $whoami: $message\0";
 
     # it's possible that we'll get an error from sending
     # (e.g. if method is UDP and there is no UDP listener,
@@ -662,9 +958,9 @@ sub connection_ok {
 }
 
 sub disconnect {
-    close SYSLOG;
     $connected = 0;
     $syslog_send = undef;
+    return close SYSLOG;
 }
 
 1;
